/*
 * Buffered UART logger.
 * - Support multiple UARTs reading threads
 *    - can temporarily disable UART (e.g. for SW update)
 * - Circular buffer for logging (with src info added)
 * - Unix Socket based reading
 *    - multiple simultaneous (independant) read sessions
 *           - session aging (inactive sessions get closed)
 *    - filters (single or all uarts, raw text or html)
 *    - read position reset capabilities (refresh full)
 *    - Logs can be added via socket
 */
#include <stdio.h>
#include <iostream>
#include <unistd.h>
#include "Uart.h"
#include "Socket.h"
#include "Buffer.h"

using namespace std;

static const char *TTY_NAME[] = { "/dev/ttyAMA0", "/dev/ttyUSB0", "/dev/ttyUSB1" };
Buffer buf;


class TestUart : public Uart {
 public:
  TestUart(const char *dev, int speed, int parity, bool hwFlowCtrl)
    : Uart(dev, speed, parity, hwFlowCtrl) {};
  //~TestUart(void) {}
 protected:
  void received(const char *tmp) {
    buf.put(tmp);
    return;
    int sz = *tmp++ + 1;
    for (int i=0; i<sz; i++)
      switch (tmp[i]) {
      case 0: printf("<0>"); break;
      case '\n': printf("<NL>\n"); break;
      case '\r': printf("<CR>"); break;
      default:
        if ((tmp[i] > 31) && (tmp[i] <128))
          putchar(tmp[i]);
        else
          printf("<x%d>", tmp[i]);
      break;
    }
    //printf("\n");
  }
};

// we do support up to 7 UART interfaces, dynamically started/stopped
TestUart *uart[7];

class PilotSocketServer : public SocketServer {
 public:
  PilotSocketServer(void) : SocketServer("/usr/local/tmp/tobe-pilotd.sock") {}
 protected:
  virtual int doCmd(const char *cmd, int fd) {
    //cout << "Received: " << cmd << '\n';
    if (strcmp(cmd, "read") == 0) {
      char tmp[9];
      while (buf.get(0, tmp))
        send(fd, tmp+1, (7 & *tmp) +1);
    } else if (strncmp(cmd, "send:0:", 7) == 0) {
      char tmp[64];
      snprintf(tmp, 64, "%s\r\n", cmd+7);
      if (uart[0]) uart[0]->send(tmp);
    } else if (strncmp(cmd, "start:0", 7) == 0) {
      if (uart[0] == NULL) uart[0] = new TestUart(TTY_NAME[0], 115200, 0, false);
    } else if (strncmp(cmd, "stop:0", 6) == 0) {
      if (uart[0] != NULL) {
        delete uart[0];
        uart[0] = NULL;
      }
    }
    return 0;
  }
};


int main(void) {
  cout << "tobe-pilotd: started\n";
  PilotSocketServer sockSvr;

  sockSvr.run();

  cout << "tobe-pilotd: done.\n";
  return 0;
}
